<?php
namespace PME\Controllers\RestricoesContratuais;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class GrupoPratoController {
    
  protected $entityManager;
  protected $newCode;
  protected $environment;
  protected $connection;
  
  public function __construct($entityManager, $connection, $newCode, $environment) {
  	$this->entityManager = $entityManager;
  	$this->newCode = $newCode;
  	$this->environment = $environment;
  	$this->connection = $connection;
  }
  
  public function excluiGrupo(DTO\Request $request, DTO\Response $response) {
		try {		
			$params = $request->getParameters()["checkedRows"];
            $this->entityManager->getConnection()->beginTransaction();
			
			foreach ($params as $param) {
				$validacao = $this->validaPratosGrupo($param);
                
				if ($validacao !== true) {
					$this->entityManager->getConnection()->rollBack();
					$response->addDataset(new DataSet('mensagem', array($validacao)));
					return;
				};
				
				$sql = "DELETE FROM GRUPOPRATOPAE 
                         WHERE CDFILCENTPME = :CDFILCENTPME 
                           AND CDTIPOCARD = :CDTIPOCARD
                           AND CDSERVFILI = :CDSERVFILI
                           AND NRAGRUPGRPRAT = :NRAGRUPGRPRAT
                           AND NRGRUPOPRATOPAE = :NRGRUPOPRATOPAE
                            AND NRORG = :NRORG";
                
                $params = array(
                    'CDFILCENTPME' => $param['CDFILCENTPME'],
                    'CDTIPOCARD' => $param['CDTIPOCARD'],
                    'CDSERVFILI' => $param['CDSERVFILI'],
                    'NRAGRUPGRPRAT' => $param['NRAGRUPGRPRAT'],
                    'NRGRUPOPRATOPAE' => $param['NRGRUPOPRATOPAE'],
                    'NRORG' => $this->environment->getNrOrg()
                );
                
                $this->connection->prepare($sql)->execute($params);
                
			}
			
            $this->entityManager->getConnection()->commit();
        }catch (\Exception $e) {
           $this->entityManager->getConnection()->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function validaPratosGrupo($param){
		// Valida se grupo tem pratos
		$sql = "SELECT I.NRITGRUPOPRATO, I.CDFILAUXPRAT, I.CDPRATO
                  FROM ITGRUPOPRATOPAE I
                 WHERE I.NRGRUPOPRATOPAE = :NRGRUPOPRATOPAE";
        
        $paramSql = array(
            'NRGRUPOPRATOPAE' => $param['NRGRUPOPRATOPAE']
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_TEM_PRATO';
        }
        
        // Valida se grupo possui item da frequência
        $sql = "SELECT NRITFREQGRPRATMESPAE FROM ITFREQGRPRATMESPAE I
                 WHERE I.NRGRUPOPRATOPAE = :NRGRUPOPRATOPAE";
      
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_TEM_ITEM_FREQUENCIA';
        }
        
        // Valida se grupo tem item de restrição contratrual
        $sql = "SELECT NRITRESTPRODGRPRAT FROM ITRESTPRODGRPRAT I
                 WHERE I.NRGRUPOPRATOPAE = :NRGRUPOPRATOPAE";
       
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_TEM_ITEM_RESTRICAO';
        }
        
        return true;
	}
	
	public function salvaGrupo(DTO\Request $request, DTO\Response $response) {
		try {			
			$params = $request->getParameters()["editedRows"];
            $this->entityManager->getConnection()->beginTransaction();
			
			foreach ($params as $param) {
				if ($param['__is_new'] === true) {
					$validacao = $this->validaGrupo($param);
					
					if ($validacao !== true) {
						$this->entityManager->getConnection()->rollBack();
						$response->addDataset(new DataSet('mensagem', array($validacao)));
						return;
					};
				}
				
			    $maxGrupoPrato = $this->getMaxGrupoPrato($param);
				
				// Insere grupo no agrupamento
				$sql = "INSERT INTO GRUPOPRATOPAE (NRGRUPOPRATOPAE, CDFILCENTPME, CDTIPOCARD, CDSERVFILI,
				            CDGRUPOPRATOPAE, NMGRUPOPRATOPAE, NRAGRUPGRPRAT, NRORG)
				        VALUES (:NRGRUPOPRATOPAE, :CDFILCENTPME, :CDTIPOCARD, :CDSERVFILI, :CDGRUPOPRATOPAE,
				            :NMGRUPOPRATOPAE, :NRAGRUPGRPRAT, :NRORG)";
                
                $paramsIn = array(
                    'NRGRUPOPRATOPAE' => $maxGrupoPrato['NRGRUPOPRATOPAE'] + 1,
                    'CDFILCENTPME' => $param['CDFILCENTPME'],
                    'CDTIPOCARD' => $param['CDTIPOCARD'],
                    'CDSERVFILI' => $param['CDSERVFILI'],
                    'CDGRUPOPRATOPAE' => str_pad(intval($maxGrupoPrato['CDGRUPOPRATOPAE']) + 1, 10, "0", STR_PAD_LEFT),
                    'NMGRUPOPRATOPAE' => $param['NMGRUPOPRATOPAE'],
                    'NRAGRUPGRPRAT' => $param['NRAGRUPGRPRAT'],
                    'NRORG' => $this->environment->getNrOrg()
                );
                
                $this->entityManager->getConnection()->prepare($sql)->execute($paramsIn);
			}
			
            $this->entityManager->getConnection()->commit();
        }catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function getMaxGrupoPrato($param){
	    $sql = "SELECT MAX(NRGRUPOPRATOPAE) NRGRUPOPRATOPAE 
                  FROM GRUPOPRATOPAE G";
    
        $nrGrupoPrato = $this->entityManager->getConnection()->fetchAssoc($sql)['NRGRUPOPRATOPAE'];
        
        $sql = "SELECT MAX(CDGRUPOPRATOPAE) CDGRUPOPRATOPAE 
                  FROM GRUPOPRATOPAE G
                  WHERE CDTIPOCARD = :CDTIPOCARD
                  AND CDSERVFILI = :CDSERVFILI
                  AND CDFILCENTPME = :CDFILCENTPME";
        
        $params = array(
            'CDTIPOCARD' => $param['CDTIPOCARD'],
            'CDSERVFILI' => $param['CDSERVFILI'],
            'CDFILCENTPME' => $param['CDFILCENTPME'],
        );
        
        $cdGrupoPrato = $this->entityManager->getConnection()->fetchAssoc($sql, $params)['CDGRUPOPRATOPAE'];
        
        return ['NRGRUPOPRATOPAE' => $nrGrupoPrato, 'CDGRUPOPRATOPAE' => $cdGrupoPrato];
	}
	
	private function validaGrupo($param){
		// Valida se grupo já é cadastrado no agrupamento
		$sql = "SELECT G.NRGRUPOPRATOPAE, G.CDFILCENTPME, G.CDTIPOCARD, G.CDSERVFILI,
                	   G.NRAGRUPGRPRAT, AG.NMAGRUPGRPRAT, G.CDGRUPOPRATOPAE, G.NMGRUPOPRATOPAE
                 FROM GRUPOPRATOPAE G
                INNER JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = G.NRAGRUPGRPRAT
                WHERE G.CDFILCENTPME = :CDFILCENTPME
                  AND G.CDTIPOCARD = :CDTIPOCARD
                  AND G.CDSERVFILI = :CDSERVFILI
                  AND G.NRAGRUPGRPRAT = :NRAGRUPGRPRAT
                  AND G.NMGRUPOPRATOPAE = :NMGRUPOPRATOPAE
                   AND G.NRORG = :NRORG";
        
        $paramSql = array(
            'CDFILCENTPME' => $param['CDFILCENTPME'],
            'CDTIPOCARD' => $param['CDTIPOCARD'],
            'CDSERVFILI' => $param['CDSERVFILI'],
            'NRAGRUPGRPRAT' => $param['NRAGRUPGRPRAT'],
            'NMGRUPOPRATOPAE' => $param['NMGRUPOPRATOPAE'],
            'NRORG' => $this->environment->getNrOrg()
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_GRUPO_CADASTRADO:' . $resultado[0]['NMAGRUPGRPRAT'];
        }
        
        return true;
	}
	
	public function salvaPrato(DTO\Request $request, DTO\Response $response) {
		$this->entityManager->getConnection()->beginTransaction();
		try {
		    $params = $request->getParameters()["editedRows"];
            
			foreach ($params as $param) {
			    $pratos = array();
			    if(is_array($param['CDPRATO'])) {
			        $i = 0;
			        foreach($param['CDPRATO'] as $cdprato){
			            $pratos[] = array(
			                            'CDPRATO' => $cdprato,
			                            'CDFILAUXPRAT' => $param['CDFILAUXPRAT'][$i],
			                            'CDSERVFILI' => $param['CDSERVFILI']
			                        );
			             $i++;
			        }
			    }
			    $errors = array();
			    foreach ($pratos as $prato) {
    				if ($param['__is_new'] === true) {
    					$validacao = $this->validaPratos($prato);
    					
    					if ($validacao !== true) {
    				        $errors[] = $validacao;
    				        continue;
    					}
    				}
    				
    			    $nrItGrupoPrato = $this->getMaxNrItGrupPrato()['NRITGRUPOPRATO'];
    				
    				// Insere grupo no agrupamento
    				$sql = "INSERT INTO ITGRUPOPRATOPAE (NRITGRUPOPRATO, NRGRUPOPRATOPAE, CDFILAUXPRAT, CDPRATO, NRORG)
    				        VALUES (:NRITGRUPOPRATO, :NRGRUPOPRATOPAE, :CDFILAUXPRAT, :CDPRATO, :NRORG)";
                    
                    $paramsIn = array(
                        'NRITGRUPOPRATO' => $nrItGrupoPrato + 1,
                        'NRGRUPOPRATOPAE' => $param['NRGRUPOPRATOPAE'],
                        'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                        'CDPRATO' => $prato['CDPRATO'],
                        'NRORG' => $this->environment->getNrOrg()
                    );
                    
                    $this->entityManager->getConnection()->prepare($sql)->execute($paramsIn);
			    }
			}
			
            $this->entityManager->getConnection()->commit();
            
            if(!empty($errors)) {
                $returnError = 'ERROR_PRAT_CAD: '.implode(", ", $errors);
    			$response->addDataset(new DataSet('mensagem', array($returnError)));
    			return;
            }
        }catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function getMaxNrItGrupPrato(){
	    $sql = "SELECT MAX(NRITGRUPOPRATO) NRITGRUPOPRATO
                  FROM ITGRUPOPRATOPAE I";
    
        $resultado = $this->entityManager->getConnection()->fetchAssoc($sql);
        return $resultado;
	}
	
	private function validaPratos($param){
		// Valida se o prato está cadastrado em algum grupo do serviço
		$sql = "SELECT I.NRITGRUPOPRATO, I.CDFILAUXPRAT, I.CDPRATO, G.NMGRUPOPRATOPAE, P.NMPRATO
                  FROM ITGRUPOPRATOPAE I
                  JOIN GRUPOPRATOPAE G 
                  ON I.NRGRUPOPRATOPAE = G.NRGRUPOPRATOPAE
                  INNER JOIN PRATO P 
                  ON P.CDPRATO = I.CDPRATO
				   AND P.CDFILAUXPRAT = I.CDFILAUXPRAT
                 WHERE I.CDPRATO = :CDPRATO
                 AND G.CDSERVFILI = :CDSERVFILI";
        
        $paramSql = array(
            'CDPRATO' => $param['CDPRATO'],
            'CDSERVFILI' => $param['CDSERVFILI']
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return $resultado[0]['NMPRATO'];
        }
        
        return true;
	}
}