import {
  ModalService, Modal, Form, Select, Grid, IButtonEvent, Button, DialogService, GridEditable, IComponentRender, Radio,
} from '@zeedhi/common';
import {
  Loader, Metadata, IEventParam, Http, IDictionary, I18n, Config,
} from '@zeedhi/core';
import { PmeMask } from '@/config';

export class Pla01308UtilizacaoProdutosController {
  private utilities = Loader.getInstance('Utilities');
  
  private appController = Loader.getInstance('AppController');

  public modalUtilizacaoProdutos!: Modal;
  
  public prato = Loader.getInstance('Pla01300PratoController');
  
  public receita = Loader.getInstance('ReceitaController');
  
  public configCalculate({ row, component }: IDictionary) {
    const formUtilizacaoProdutos: Form = Metadata.getInstance('formFiltroUtilizacaoProd');
    const gridParent = component.parent;
    let typeFieldVR = '';
    let typeFieldQT = '';
    let required = '';
    
    if (formUtilizacaoProdutos) {
      typeFieldVR = 'PR';
      typeFieldQT = 'PRAT';
      required = 'PRUNR';
    }

    const percentualChange = this.receita.calculateFields(row, component, typeFieldVR, typeFieldQT, required);

    if (this.receita.propChange) {
      gridParent.datasource.data.forEach((e: IDictionary) => {
        if (row.CDPRODUTO !== e.CDPRODUTO && e[`IDQTDEFIX${typeFieldVR}`] !== 'S') {
          this.receita.changeProportional(e, typeFieldVR, typeFieldQT, percentualChange);
        }
      });
    }
    
    // row.NMPRODUTO = gridParent.NMPRODUTO.value;
    // row.NMPRODUTO = row.NMPRODUTOAUX;
    // gridParent.updateRow(row);
  }
  
  private async createModalUtilizacaoProdutos() {
    this.modalUtilizacaoProdutos = await ModalService.createFromJson(
      'planejamento/prato/modals/pla01308_utilizacao_produtos', true,
    );
  }
  
  public closeModalUtilizacaoProdutos() {
    this.modalUtilizacaoProdutos.hide();
  }
  
  public openModalUtilizacaoProdutos() {
    if (!this.modalUtilizacaoProdutos) {
      this.createModalUtilizacaoProdutos().then(() => {
        this.modalUtilizacaoProdutos.show();
      });
    } else {
      this.modalUtilizacaoProdutos.show();
    }
  }

  public fullscreen({ component }: IButtonEvent) {
    this.modalUtilizacaoProdutos.fullscreen = !this.modalUtilizacaoProdutos.fullscreen;
    component.iconName = (this.modalUtilizacaoProdutos.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }

  public inicContainer() {
    const project = localStorage.getItem('PRODUCT_ID');
    
    const tpReceitaCard = Metadata.getInstance<Select>('TPRECEITA');
    const tpReceita = Metadata.getInstance<Select>('TPRECEITA1');
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');

    if (project === '4875') {
      tpReceitaCard.isVisible = true;
      tpReceita.isVisible = false;
    } else {
      tpReceitaCard.isVisible = false;
      tpReceita.isVisible = true;
      cardapio.isVisible = false;
    }
  }
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/find/produtoNiveisEstoque',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
  
  public setFilterService({ component }: IEventParam<Select>) {
    const form: Form = Metadata.getInstance('formFiltroUtilizacaoProd');
    const row = form.value;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: row.UNIDADE,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: localStorage.getItem('PRODUCT_ID') === '4875' ? '' : 'N',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public changeUnityServ() {
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    const serv = Metadata.getInstance<Select>('SERVICO');
    serv.setValue('');
    grid.datasource.data = [];

    this.setFilterUtilizacaoProdutos();
  }
  
  public setFilterUnidade({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value : 'T';
    
    const filter = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterUtilizacaoProdutos() {
    const form: Form = Metadata.getInstance('formFiltroUtilizacaoProd');
    const row = form.value;
    let tpReceitaCard = Metadata.getInstance<Select>('TPRECEITA');
    const tpReceita = Metadata.getInstance<Select>('TPRECEITA1');
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    row.NRLISTASERV = '';
    
    if (tpReceitaCard === null) tpReceitaCard = tpReceita;

    if (this.utilities.returnPrat(row.PRATOINICIAL.CDPRATO, row.PRATOFINAL.CDPRATO) && row.PRATOFINAL.CDPRATO !== null) {
      this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
      row.PRATOFINAL = null;
    }
    
    if (tpReceitaCard.value || tpReceita.value) {
      Http.post('/prato/retornaListaServ', { requestType: 'Row', row }).then((response) => {
        const result = response.data.dataset.result.data;
        row.NRLISTASERV = result;
        const conditions = [
          {
            name: 'P_CDPRATOINI',
            operator: '=',
            value: row.PRATOINICIAL.CDPRATO,
          },
          {
            name: 'P_CDPRATOFIN',
            operator: '=',
            value: row.PRATOFINAL.CDPRATO,
          },
          {
            name: 'P_CDFILAUXPRAT',
            operator: '=',
            value: 'P',
          },
          {
            name: 'P_CDPRODUTO',
            operator: '=',
            value: row.PRODINI ? row.PRODINI : 'T',
          },
          {
            name: 'P_CDFILCENTPME',
            operator: '=',
            value: this.appController.getCdCentral(),
          },
          {
            name: 'P_RECEFILI',
            operator: '=',
            value: tpReceitaCard.indexOf('L') === -1 ? 'N' : 'S',
          },
          {
            name: 'P_RECETPCARD',
            operator: '=',
            value: tpReceitaCard.indexOf('T') === -1 ? 'N' : 'S',
          },
          {
            name: 'P_RECEPADR',
            operator: '=',
            value: tpReceitaCard.indexOf('P') === -1 ? 'N' : 'S',
          },
          {
            name: 'P_RECESERV',
            operator: '=',
            value: tpReceitaCard.indexOf('S') === -1 ? 'N' : 'S',
          },
          {
            name: 'TEXTO_PADRAO',
            operator: '=',
            value: I18n.translate('pme::padrao'),
          },
          {
            name: 'TEXTO_TPCARD',
            operator: '=',
            value: I18n.translate('pme::tipoDeCardapio'),
          },
          {
            name: 'P_CDTIPOCARD',
            operator: '=',
            value: row.TIPOCARD ? row.TIPOCARD : 'T',
          },
          {
            name: 'P_NRLISTASERV',
            operator: '=',
            value: result,
          },
        ];
        grid.datasource.filter = conditions;
        grid.datasource.get();
      });
    }
  }
  
  public confirmDeletePlate({ component }: IEventParam<Button>) {
    const nameButton = component.name;
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    let editedRows: any = [];
    
    if (nameButton === 'excluirUtilizacaoProd') {
      const form: Form = Metadata.getInstance('formFiltroUtilizacaoProd');
      const row = form.value;
      
      editedRows[0] = {
        CDFILAUXPRAT: row.CDFILAUXPRAT,
        CDFILIAL: row.CDFILIAL,
        CDNVPRATO: row.CDNVPRATO,
        CDPRATO: row.CDPRATO,
      };
    } else {
      editedRows = grid.selectedRows;
    }
    
    if (editedRows.length === 0) {
      this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    } else {
      DialogService.show({
        name: nameButton,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.excluir(grid, editedRows) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public excluir(grid: Grid, editedRows: any) {
    const buttonYes = Metadata.getInstance<Button>('yes');
    buttonYes.disabled = true;

    Http.post('/', { editedRows }).then(() => {
      this.utilities.showAlert('Alert', 'pme::excluidoComSucesso', 'success');
      grid.selectedRows = [];
      grid.reload();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      // if (error.includes('ORA-02292')) this.utilities.showAlert('alert', 'pme::ERROR_REGISTROFILHO', 'error');
    }).finally(() => {
      DialogService.hide();
      buttonYes.disabled = false;
    });
  }
  
  public atualizarGrid() {
    const grid: GridEditable = Metadata.getInstance('gridUtilizacaoProd');
    const editedRows = grid.getEditedRows() as IDictionary;
    const produto = editedRows[0].CDARVPROD;
    const nmproduto = editedRows[0].NMPRODUTO;
    grid.datasource.currentRow.CDARVPROD = produto;
    grid.datasource.currentRow.NMPRODUTO = nmproduto;
    grid.editing = false;

    const toolbarSlot = grid.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = !e.disabled;
      if (e.name === 'saveButton') e.disabled = !e.disabled;
    });
    
    this.sim();
  }
  
  public salvarGrid() {
    const grid: GridEditable = Metadata.getInstance('gridUtilizacaoProd');
    const editedRows = grid.getEditedRows() as IDictionary;
    const produto = editedRows[0].CDARVPROD;
    const nmproduto = editedRows[0].NMPRODUTO;
    grid.datasource.currentRow.CDARVPROD = produto;
    grid.datasource.currentRow.NMPRODUTO = nmproduto;
    if (editedRows.length > 0) {
      this.atualizarGrid();
    }
  }
  
  public confirmar({ component }: IEventParam<Button>) {
    const form: Form = Metadata.getInstance('formFiltroUtilizacaoProd');
    const row = form.value;
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    const rowsGrid = grid.selectedRows;
    let padrao!: string;

    if (row.TPRECEITA === null) row.TPRECEITA = row.TPRECEITA1;

    if (rowsGrid.length > 0) {
      row.TPRECEITA.forEach((rows: any) => {
        if (rows === 'P') {
          padrao = 'P';
        }
      });
      
      if (padrao === 'P') {
        if (this.prato.operatorValid !== false) this.editar();
        else {
          this.utilities.showAlert('Alert', 'pme::modificacaoRecPadrao', 'warning');
          this.prato.openModalGroupOperador(component);
        }
      } else this.editar();
    } else this.utilities.showAlert('Alert', 'pme::aoMenosUmPratoSelecionado', 'error');
  }
  
  public editar() {
    DialogService.show({
      name: 'finalizar',
      type: 'warning',
      title: 'pme::atencao',
      text: 'pme::editarUtilizacaoProdutos',
      buttons: [
        {
          name: 'salvar',
          component: 'ZdButton',
          label: 'pme::sim',
          events: { click: () => this.sim() },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'pme::nao',
          events: { click: () => DialogService.hide() },
        },
        {
          name: 'progressLine',
          component: 'ZdProgress',
          label: 'Progress',
          indeterminate: true,
          cssClass: 'loading-progress-line',
        },
      ],
    });
  }
  
  public sim() {
    this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
    DialogService.hide();
  }
  
  public alteracaoPercentualQuantidade() {
    const alteracao = Metadata.getInstance<Radio>('ALTERACAO');
    const grid = Metadata.getInstance<GridEditable>('gridUtilizacaoProd');
    if (alteracao.value === '1') {
      grid.columns[11].label = I18n.translate('pme::quantidadePerc');
      grid.columns[12].label = I18n.translate('pme::qtBrutaPerc');
    } else {
      grid.columns[11].label = I18n.translate('pme::quantidade');
      grid.columns[12].label = I18n.translate('pme::qtdeBruta');
    }
  }
}
